@@ -54,10 +54,12 @@ from paramiko.pkey import PKey
 from paramiko.rsakey import RSAKey
 from paramiko.ssh_exception import SSHException
 from waitress import serve
+from waitress.adjustments import Adjustments
 from werkzeug.contrib.cache import BaseCache, SimpleCache
 from werkzeug.exceptions import BadRequest, Forbidden, HTTPException, NotFound
 from werkzeug.routing import BaseConverter, ValidationError
 from werkzeug.utils import html
+from werkzeug.contrib.fixers import ProxyFix
 
 from .identity import Identity
 from .keystore import (DuplicatePublicKeyError, KeyStore, KeyTypeError,
@@ -77,7 +79,7 @@ __all__ = ('AUTHORIZATION_TIMEOUT',
            'get_key_store', 'get_master_key_store', 'get_public_key',
            'get_remote_set', 'get_team', 'get_token_store',
            'list_public_keys', 'main', 'main_parser', 'master_key',
-           'public_key', 'remote_dict', 'server_version')
+           'public_key', 'remote_dict', 'server_version', 'token')
 
 
 #: (:class:`datetime.timedelta`) How long does each temporary authorization
@@ -376,6 +378,41 @@ def get_identity(token_id: str) -> Identity:
     raise HTTPException(response=response)
 
 
+@app.route('/tokens/<token_id:token_id>/')
+@typed
+def token(token_id: str):
+    """The owner identity that the given token holds if the token is
+    authenticated.  Otherwise it responds :http:statuscode:`403`,
+    :http:statuscode:`404`, :http:statuscode:`410`, or
+    :http:statuscode:`412`.  See also :func:`get_identity()`.
+
+    .. code-block:: http
+
+       GET /tokens/0123456789abcdef/ HTTPS/1.1
+       Accept: application/json
+
+    .. code-block:: http
+
+       HTTPS/1.0 200 OK
+       Content-Type: application/json
+
+       {
+         "identifier": "dahlia",
+         "team_type": "geofront.backends.github.GitHubOrganization"
+       }
+
+    :param token_id: the token id that holds the identity
+    :type token_id: :class:`str`
+    :status 200: when the token is authenticated
+
+    """
+    identity = get_identity(token_id)
+    return jsonify(
+        team_type='{0.__module__}.{0.__qualname__}'.format(identity.team_type),
+        identifier=identity.identifier
+    )
+
+
 def get_master_key_store() -> MasterKeyStore:
     """Get the configured master key store implementation.
 
@@ -829,6 +866,12 @@ def main_parser() -> argparse.ArgumentParser:  # pragma: no cover
                         action='store_true',
                         help='renew the master key before the server starts. '
                              'implies --create-master-key option')
+    parser.add_argument('--trusted-proxy',
+                        action='store_true',
+                        help='IP address of a client allowed to override '
+                             'url_scheme via the X-Forwarded-Proto header. '
+                             'useful when it runs behind reverse proxy. '
+                             '-d/--debug option disables this option')
     parser.add_argument('-d', '--debug',
                         action='store_true',
                         help='debug mode.  note that this option may make '
@@ -882,11 +925,21 @@ def main():  # pragma: no cover
             master_key_store,
             master_key_renewal_interval
         )
+    waitress_options = {}
+    if args.trusted_proxy:
+        if hasattr(Adjustments, 'trusted_proxy'):
+            # > 0.8.8
+            # https://github.com/Pylons/waitress/pull/42
+            waitress_options['trusted_proxy'] = True
+        else:
+            # <= 0.8.8
+            app.wsgi_app = ProxyFix(app.wsgi_app)
     try:
         if args.debug:
             app.run(args.host, args.port, debug=True)
         else:
-            serve(app, host=args.host, port=args.port, asyncore_use_poll=True)
+            serve(app, host=args.host, port=args.port, asyncore_use_poll=True,
+                  **waitress_options)
     finally:
         if master_key_renewal_interval is not None:
             master_key_renewal.terminate()
